import { CSSProperties, FC, ReactNode } from "react";
import { Language } from "../types/Language";
import { ConventionEdition } from "../types/ConventionEdition";
import { EventActivity } from "../types/EventActivity";
import { UITranslation } from "../types/UITranslation";
import { sprintf } from "sprintf-js";
import { langDateFormatter, FormattingStyle } from "../lib/langDateFormatter";
import { textualJoiner, textualColoredJoiner } from "../lib/textualJoiner";
import { vecDistance } from "../lib/vecDistance";
import Image from "next/image";
import { LanguageUI } from "../lib/flags";
import { compare } from "../lib/compare";
import { Event } from "./Event";

type DetailProps = {
  UIStrings: Record<Language, UITranslation>;
  lang: Language;
  conventionEdition: ConventionEdition;
  mediaPrefix: string;
  selectedEvent: EventActivity;
  lightDarkColors: string[];
};

export const Detail: FC<DetailProps> = ({
  UIStrings,
  lang,
  conventionEdition,
  mediaPrefix,
  selectedEvent,
  lightDarkColors,
}) => {
  const backdrop =
    (selectedEvent.picture.uuid === null
      ? null
      : `${mediaPrefix}${selectedEvent.picture[lang]}`) ??
    (conventionEdition.image_default_event.uuid === null
      ? null
      : `${mediaPrefix}${conventionEdition.image_default_event[lang]}`) ??
    undefined;
  const style_: CSSProperties = {};
  if (backdrop !== undefined) style_.backgroundImage = `url(${backdrop})`;
  const places = Object.fromEntries(
    conventionEdition.places.map((x) => [x.uuid, x])
  );
  const categories = Object.fromEntries(
    conventionEdition.tags.map((x) => [x.uuid, x])
  );
  const registrationTiers = Object.fromEntries(
    conventionEdition.registration_tiers.map((x) => [x.uuid, x])
  );
  const keypairs: [string, ReactNode][] = [];
  keypairs.push([
    UIStrings[lang].schedule_event_key_activity,
    selectedEvent.title[lang],
  ]);
  if (selectedEvent.subtitle.uuid !== null)
    keypairs.push([
      UIStrings[lang].schedule_event_key_detail,
      selectedEvent.subtitle[lang],
    ]);
  keypairs.push([
    UIStrings[lang].schedule_event_key_when,
    sprintf(
      UIStrings[lang].schedule_event_key_when_formatted,
      langDateFormatter(
        FormattingStyle.LongDayOfWeekWithNumber,
        lang,
        conventionEdition.timezone,
        selectedEvent.time_start
      ),

      langDateFormatter(
        FormattingStyle.HoursAndMinutes,
        lang,
        conventionEdition.timezone,
        selectedEvent.time_start
      ),
      langDateFormatter(
        FormattingStyle.HoursAndMinutes,
        lang,
        conventionEdition.timezone,
        selectedEvent.time_end
      )
    ),
  ]);
  if (selectedEvent.places.length > 0)
    keypairs.push([
      selectedEvent.places.length > 1
        ? UIStrings[lang].schedule_event_key_places
        : UIStrings[lang].schedule_event_key_place,
      textualColoredJoiner(
        lang,
        selectedEvent.places
          .map((x) => places[x])
          .sort((a, b) => a.ordering - b.ordering),
        lightDarkColors
      ),
    ]);
  if (selectedEvent.categories.length > 0)
    keypairs.push([
      selectedEvent.categories.length > 1
        ? UIStrings[lang].schedule_event_key_categories
        : UIStrings[lang].schedule_event_key_category,
      textualColoredJoiner(
        lang,
        selectedEvent.categories
          .map((x) => categories[x])
          .sort((a, b) => a.ordering - b.ordering),
        lightDarkColors
      ),
    ]);
  if (
    selectedEvent.language !== null &&
    selectedEvent.language !== conventionEdition.language
  )
    keypairs.push([
      UIStrings[lang].schedule_event_key_language,
      UIStrings[lang][LanguageUI[selectedEvent.language]],
    ]);
  if (!["", "-"].includes(selectedEvent.description[lang]))
    keypairs.push([
      UIStrings[lang].schedule_event_key_description,
      selectedEvent.description[lang],
    ]);
  if (
    !(
      selectedEvent.conbook_pages.length === 0 ||
      (selectedEvent.conbook_pages.length === 1 &&
        selectedEvent.conbook_pages[0] === 0)
    )
  )
    keypairs.push([
      selectedEvent.conbook_pages.length > 1
        ? UIStrings[lang].schedule_event_key_on_conbooks_pages
        : UIStrings[lang].schedule_event_key_on_conbooks_page,
      selectedEvent.conbook_pages.map((x) => String(x)).join(", "),
    ]);
  if (
    !(
      selectedEvent.attendable_by.length === 0 ||
      (selectedEvent.attendable_by.length === 1 &&
        registrationTiers[selectedEvent.attendable_by[0]].tier[lang] ===
          "-%auto%-")
    )
  )
    keypairs.push([
      selectedEvent.attendable_by.length > 1
        ? UIStrings[lang].schedule_event_key_elegible_registrations
        : UIStrings[lang].schedule_event_key_elegible_registration,
      textualJoiner(
        lang,
        selectedEvent.attendable_by
          .map((x) => registrationTiers[x])
          .sort((a, b) => a.level - b.level)
          .map((x) => x.tier[lang])
      ),
    ]);
  const pressBack = () => window.history.back();
  const otherEventsWithVec = conventionEdition.events.filter(
    (ev) => ev.uuid !== selectedEvent.uuid && ev.word2vec.some((x) => x !== 0.0)
  );
  const hasOwnVec = selectedEvent.word2vec.some((x) => x !== 0.0);
  const rankedEvs = !hasOwnVec
    ? []
    : otherEventsWithVec
        .map((ev) => ({
          ev,
          distance: vecDistance(selectedEvent.word2vec, ev.word2vec),
        }))
        .sort(
          (a, b) =>
            compare(a.distance, b.distance) || compare(a.ev.uuid, b.ev.uuid)
        )
        .map(({ ev }) => ev);
  return (
    <main className="detail">
      <nav style={style_}>
        <Image
          src="/icons/ic_arrow_back_black_24dp.svg"
          alt={UIStrings[lang].menu_continue_in_background}
          width={32}
          height={32}
          style={{ filter: "invert(1)" }}
          onClick={pressBack}
        />
        <h1>{selectedEvent.title[lang]}</h1>
      </nav>
      {keypairs
        .filter((x) => x.length == 2)
        .map(([k, v]) => (
          <dl key={`${k}.${v}`}>
            <dt>{k}</dt>
            <dd>{v}</dd>
          </dl>
        ))}
      {rankedEvs.length && (
        <dl>
          <dt>{UIStrings[lang].see_more}</dt>
          <dd>
            {rankedEvs.slice(0, 5).map((ev) => (
              <Event
                key={ev.uuid}
                UIStrings={UIStrings}
                lang={lang}
                event={ev}
                conventionEdition={conventionEdition}
                places={places}
                showDay
              />
            ))}
          </dd>
        </dl>
      )}
    </main>
  );
};
