import { FC } from "react";
import { Language } from "../types/Language";
import { ConventionEdition } from "../types/ConventionEdition";
import { UITranslation } from "../types/UITranslation";
import {
  AppPageState,
  appStateSerialize,
  appStateParse,
  AppState,
} from "../lib/appState";
import { clsx } from "clsx";
import {
  FormattingStyle,
  dateRecomposer,
  recomposedDateFormatter,
} from "../lib/langDateFormatter";
import { MasterFragmentSummary } from "./MasterFragmentSummary";
import { MasterFragmentDay } from "./MasterFragmentDay";
import { MasterFragmentNotification } from "./MasterFragmentNotification";
import { MasterFragmentRules } from "./MasterFragmentRules";
import { MasterFragmentSocialMedia } from "./MasterFragmentSocialMedia";
import { ConventionSeries } from "../types/ConventionSeries";
import { MasterFragmentUrgentMessage } from "./MasterFragmentUrgentMessage";
import { FontAwesomeIcon } from "@fortawesome/react-fontawesome";
import { faLanguage, faMoon, faSun } from "@fortawesome/free-solid-svg-icons";
import { DayNightState } from "../types/DayNightState";

type MasterProps = {
  UIStrings: Record<Language, UITranslation>;
  conventionSeries: ConventionSeries;
  conventionEdition: ConventionEdition;
  mediaPrefix: string;
  rerender: () => void;
  flipDayNight: () => void;
  dayNight: DayNightState;
  changeLanguage: () => void;
};

export const Master: FC<MasterProps> = ({
  UIStrings,
  conventionSeries,
  conventionEdition,
  rerender: rerender2,
  flipDayNight,
  dayNight,
  changeLanguage,
}) => {
  const rerender = () => rerender2();
  const defaultPageData: AppState = [
    conventionEdition.language,
    AppPageState.master,
    0,
    conventionEdition.uuid,
  ];
  const [lang, page, tab, identifier] =
    appStateParse(window.location.hash.substring(1)) ?? defaultPageData;
  const dateDays = [
    ...new Set(
      conventionEdition.events.map((x) => {
        const d = dateRecomposer(conventionEdition.timezone, x.time_start);
        d.setHours(0);
        d.setMinutes(0);
        return d.getTime();
      })
    ),
  ]
    .sort()
    .map((x) => new Date(x));
  const conventionSeries2 = conventionSeries;
  const tabs: [string, FC][] = [
    [
      UIStrings[lang].schecule_tab_summary,
      () => (
        <MasterFragmentSummary
          UIStrings={UIStrings}
          conventionSeries={conventionSeries2}
          conventionEdition={conventionEdition}
          dayCount={dateDays.length}
          rerender={rerender}
        />
      ),
    ],
    ...((conventionSeries.urgent_message === null
      ? []
      : [
          [
            conventionSeries.urgent_message.tab.name[lang],
            () => (
              <MasterFragmentUrgentMessage
                lang={lang}
                urgentMessage={conventionSeries.urgent_message!}
              />
            ),
          ],
        ]) satisfies [string, FC][]),
    ...(dateDays.map((x) => [
      recomposedDateFormatter(
        FormattingStyle.DayOfWeekNumberNoParenthesis,
        lang,
        x
      ),
      () => (
        <MasterFragmentDay
          UIStrings={UIStrings}
          conventionEdition={conventionEdition}
          day={x}
        />
      ),
    ]) satisfies [string, FC][]),
    [
      UIStrings[lang].past_notifications_tab,
      () => (
        <MasterFragmentNotification
          conventionSeries={conventionSeries2}
          conventionEdition={conventionEdition}
        />
      ),
    ],
    [
      UIStrings[lang].rules,
      () => (
        <MasterFragmentRules
          UIStrings={UIStrings}
          conventionSeries={conventionSeries2}
          conventionEdition={conventionEdition}
        />
      ),
    ],
    [
      UIStrings[lang].social_media,
      () => (
        <MasterFragmentSocialMedia
          conventionSeries={conventionSeries2}
          conventionEdition={conventionEdition}
        />
      ),
    ],
  ];
  if (tab < 0 || tab >= tabs.length) {
    window.location.replace(`#${appStateSerialize(defaultPageData)}`);
    rerender();
  }
  const changeTab = (i: number) => {
    window.location.replace(
      `#${appStateSerialize([lang, page, i, identifier])}`
    );
    rerender();
  };
  const Fragment = tabs[tab][1];
  return (
    <main className="master">
      <nav>
        <div>
          <div>
            <h1>{conventionEdition.name[lang]}</h1>
            <h2>{conventionEdition.theme[lang]}</h2>
          </div>
          <div>
            <FontAwesomeIcon
              size="2x"
              icon={dayNight === DayNightState.night ? faSun : faMoon}
              onClick={flipDayNight}
            />
            {conventionSeries.languages.length > 1 && (
              <FontAwesomeIcon
                size="2x"
                icon={faLanguage}
                onClick={changeLanguage}
              />
            )}
          </div>
        </div>
        <ul>
          {tabs.map((x_, i) => {
            const x = x_[0];
            return (
              <li
                key={x}
                className={clsx(i == tab && "active")}
                onClick={() => changeTab(i)}
              >
                {x}
              </li>
            );
          })}
        </ul>
      </nav>
      <Fragment />
    </main>
  );
};
