import { ConventionEdition } from "../types/ConventionEdition";
import { ConventionSeries } from "../types/ConventionSeries";
import { Language } from "../types/Language";
import { UITranslation } from "../types/UITranslation";
import { FC } from "react";
import Image from "next/image";
import {
  AppPageState,
  appStateParse,
  appStateSerialize,
} from "../lib/appState";
import { sprintf } from "sprintf-js";
import {
  FormattingStyle,
  dateRecomposer,
  langDateFormatter,
} from "../lib/langDateFormatter";
import Link from "next/link";
import { compare } from "../lib/compare";
import { Event } from "./Event";

type MasterFragmentSummaryProps = {
  UIStrings: Record<Language, UITranslation>;
  conventionSeries: ConventionSeries;
  conventionEdition: ConventionEdition;
  dayCount: number;
  rerender: () => void;
};

export const MasterFragmentSummary: FC<MasterFragmentSummaryProps> = ({
  UIStrings,
  conventionSeries,
  conventionEdition,
  dayCount,
  rerender,
}) => {
  const lang =
    appStateParse(window.location.hash.substring(1))?.[0] ??
    conventionEdition.language;
  const places = Object.fromEntries(
    conventionEdition.places.map((x) => [x.uuid, x])
  );
  const goToTab = (tabId: number) => {
    window.location.replace(
      `#${appStateSerialize([
        lang,
        AppPageState.master,
        tabId,
        conventionEdition.uuid,
      ])}`
    );
    rerender();
  };
  const goToMapLink = (mapId: string) =>
    `#${appStateSerialize([lang, AppPageState.conMap, 0, mapId])}`;
  const openTime = dateRecomposer(
    conventionEdition.timezone,
    conventionEdition.ceremony_opening_time
  );
  const closeTime = dateRecomposer(
    conventionEdition.timezone,
    conventionEdition.ceremony_closing_time
  );
  const nextEdition = dateRecomposer(
    conventionEdition.timezone,
    conventionEdition.next_edition
  );
  //*
  const now = new Date();
  //*/ const now = new Date(2023, 6, 16, 13, 37); //*/
  const nowSecs = Math.floor(now.getTime() / 1000);
  const nextEditionStartForCountdown =
    now.getTime() < openTime.getTime()
      ? openTime
      : closeTime.getTime() < now.getTime() &&
        now.getTime() < nextEdition.getTime()
      ? nextEdition
      : undefined;
  const nextEditionStartForCountdownSecs =
    nextEditionStartForCountdown !== undefined
      ? Math.floor(nextEditionStartForCountdown.getTime() / 1000)
      : undefined;
  const timeUntilNextEdition =
    nextEditionStartForCountdownSecs !== undefined
      ? [
          Math.floor(
            (nextEditionStartForCountdownSecs - nowSecs) / (24 * 3600)
          ),
          Math.floor((nextEditionStartForCountdownSecs - nowSecs) / 3600) % 24,
          Math.floor((nextEditionStartForCountdownSecs - nowSecs) / 60) % 60,
        ]
      : undefined;
  const rulesTabNo =
    2 + dayCount + (conventionSeries.urgent_message !== null ? 1 : 0);
  const announcementsTabNo = rulesTabNo - 1;
  const dd = (24 * 3600 * 1000) / conventionEdition.split_day_into_n_parts;
  const nowPart = Math.floor(now.getTime() / dd);
  const endTime = Math.round((nowPart + 2) * dd);
  const eventsNow = conventionEdition.events
    .sort(
      (a, b) =>
        compare(a.time_start.formatted, b.time_start.formatted) ||
        compare(a.time_end, b.time_end) ||
        compare(
          a.title[conventionEdition.language],
          b.title[conventionEdition.language]
        )
    )
    .filter((x) => {
      const ds = dateRecomposer(conventionEdition.timezone, x.time_start);
      const de = dateRecomposer(conventionEdition.timezone, x.time_end);
      return ds.getTime() <= now.getTime() && now.getTime() < de.getTime();
    });
  const eventsNext = conventionEdition.events
    .sort(
      (a, b) =>
        compare(a.time_start.formatted, b.time_start.formatted) ||
        compare(a.time_end, b.time_end) ||
        compare(
          a.title[conventionEdition.language],
          b.title[conventionEdition.language]
        )
    )
    .filter((x) => {
      const ds = dateRecomposer(conventionEdition.timezone, x.time_start);
      return ds.getTime() > now.getTime() && ds.getTime() < endTime;
    });
  return (
    <ul className="summary">
      {conventionSeries.urgent_message && (
        <li onClick={() => goToTab(1)} style={{ cursor: "pointer" }}>
          <div className="main">
            <i
              className="icon"
              style={{
                maskImage: `url("/icons/${conventionSeries.urgent_message.card.icon}.svg")`,
                WebkitMaskImage: `url("/icons/${conventionSeries.urgent_message.card.icon}.svg")`,
              }}
            />
            <div className="info">
              <h3>{conventionSeries.urgent_message.card.name[lang]}</h3>
              <span>
                {conventionSeries.urgent_message.card.description[lang]}
              </span>
            </div>
          </div>
        </li>
      )}
      {conventionSeries.announcements.length > 0 && (
        <li
          onClick={() => goToTab(announcementsTabNo)}
          style={{ cursor: "pointer" }}
        >
          <div className="main">
            <i
              className="icon"
              style={{
                maskImage: `url("/icons/ic_bullhorn.svg")`,
                WebkitMaskImage: `url("/icons/ic_bullhorn.svg")`,
              }}
            />
            <div className="info">
              <h3>{UIStrings[lang].past_notifications}</h3>
              <span>
                {sprintf(
                  UIStrings[lang].past_notifications_last,
                  langDateFormatter(
                    FormattingStyle.TextuallyPleasantDate,
                    lang,
                    conventionEdition.timezone,
                    conventionSeries.announcements[0].modified
                  )
                )}
              </span>
            </div>
          </div>
        </li>
      )}
      {timeUntilNextEdition !== undefined && (
        <li>
          <div className="main">
            <i
              className="icon"
              style={{
                maskImage: `url("/icons/ic_timer_black_24dp.svg")`,
                WebkitMaskImage: `url("/icons/ic_timer_black_24dp.svg")`,
              }}
            />
            <div className="info">
              <h3>
                {sprintf(
                  UIStrings[lang].thingsLacking_to,
                  sprintf("%sd %sh %sm", ...timeUntilNextEdition)
                )}
              </h3>
              <span>{UIStrings[lang].for_the_beginning_of_event}</span>
            </div>
          </div>
        </li>
      )}
      <li>
        <div className="main">
          <i
            className="icon"
            style={{
              maskImage: `url("/icons/ic_access_time_black_24dp.svg")`,
              WebkitMaskImage: `url("/icons/ic_access_time_black_24dp.svg")`,
            }}
          />
          <div className="info">
            <h3>{UIStrings[lang].schedule_now_title}</h3>
            <span>
              {eventsNow.length === 0
                ? UIStrings[lang].schedule_now_no_event
                : eventsNow.length === 1
                ? sprintf(UIStrings[lang].schedule_now_n_event, 1)
                : sprintf(
                    UIStrings[lang].schedule_now_n_events,
                    eventsNow.length
                  )}
            </span>
          </div>
        </div>
        <ul className="inline-events">
          {eventsNow.map((ev) => (
            <Event
              key={ev.uuid}
              event={ev}
              UIStrings={UIStrings}
              lang={lang}
              conventionEdition={conventionEdition}
              places={places}
              showDay
            />
          ))}
        </ul>
      </li>
      <li>
        <div className="main">
          <i
            className="icon"
            style={{
              maskImage: `url("/icons/ic_av_timer_black_24dp.svg")`,
              WebkitMaskImage: `url("/icons/ic_av_timer_black_24dp.svg")`,
            }}
          />
          <div className="info">
            <h3>{UIStrings[lang].schedule_coming_title}</h3>
            <span>
              {eventsNext.length === 0
                ? UIStrings[lang].schedule_coming_no_event
                : eventsNext.length === 1
                ? sprintf(UIStrings[lang].schedule_coming_n_event, 1)
                : sprintf(
                    UIStrings[lang].schedule_coming_n_events,
                    eventsNext.length
                  )}
            </span>
          </div>
        </div>
        <ul className="inline-events">
          {eventsNext.map((ev) => (
            <Event
              key={ev.uuid}
              event={ev}
              UIStrings={UIStrings}
              lang={lang}
              conventionEdition={conventionEdition}
              places={places}
              showDay
            />
          ))}
        </ul>
      </li>
      {conventionEdition.maps.length > 0 && (
        <li>
          <div className="main">
            <i
              className="icon"
              style={{
                maskImage: `url("/icons/ic_map_black_24dp.svg")`,
                WebkitMaskImage: `url("/icons/ic_map_black_24dp.svg")`,
              }}
            />
            <div className="info">
              <h3>{UIStrings[lang].conventionmaps}</h3>
              <ul className="maps">
                {conventionEdition.maps.map((m) => (
                  <li key={m.uuid}>
                    <div className="main"></div>
                    <Link href={goToMapLink(m.uuid)}>{m.name[lang]}</Link>
                    <div className="main"></div>
                  </li>
                ))}
              </ul>
            </div>
          </div>
        </li>
      )}
      <li>
        <div className="main">
          <i
            className="icon"
            style={{
              maskImage: `url("/icons/ic_photo_camera_black_24dp.svg")`,
              WebkitMaskImage: `url("/icons/ic_photo_camera_black_24dp.svg")`,
            }}
          />
          <div className="info">
            <h3>{conventionEdition.hashtag_reminder}</h3>
            <span>{UIStrings[lang].share_your_photos_with_our_hashtag}</span>
          </div>
        </div>
      </li>
      <li onClick={() => goToTab(rulesTabNo)} style={{ cursor: "pointer" }}>
        <div className="main">
          <i
            className="icon"
            style={{
              maskImage: `url("/icons/ic_format_align_left_black_24dp.svg")`,
              WebkitMaskImage: `url("/icons/ic_format_align_left_black_24dp.svg")`,
            }}
          />
          <div className="info">
            <h3>{UIStrings[lang].read_the_rules}</h3>
            <span>{UIStrings[lang].click_here_to_open_it_masc}</span>
          </div>
        </div>
      </li>
    </ul>
  );
};
