import { Timezone } from "../types/Timezone";
import { Language } from "../types/Language";
import { DecomposedDateTime } from "../types/DecomposedDateTime";
import { sprintf } from "sprintf-js";

export const FormattingStyle = {
  // java.text.SimpleDateFormat | https://docs.oracle.com/javase/8/docs/api/java/text/SimpleDateFormat.html
  HoursAndMinutes: "HoursAndMinutes", // "hh:mm a":en | "HH:mm" -- 08:08 AM
  DayOfWeek: "DayOfWeek", // "E" -- Mon
  DayOfMonth: "DayOfMonth", // "d" -- 21
  MonthName: "MonthName", // "MMM" -- Aug
  LongDayOfWeekWithNumber: "LongDayOfWeekWithNumber", // "EEEE (d)" -- Monday (21)
  DayOfWeekNumberNoParenthesis: "DayOfWeekNumberNoParenthesis", // "E d" -- Mon 21
  TextuallyPleasantDate: "TextuallyPleasantDate", // "MMM d (E), h:mm a":en | "d MMM (E), H:mm" -- Aug 21 (Mon), 8:10 AM
  Iso: "Iso", // "yyyy-MM-dd'T'HH:mm:ss.SSSZ" -- 2023-08-21T20:10:21.683-0300
} as const;

export type FormattingStyle =
  (typeof FormattingStyle)[keyof typeof FormattingStyle];

const WEEKDAYS_FULL = {
  pt: ["Domingo", "Segunda", "Terça", "Quata", "Quinta", "Sexta", "Sábado"],
  en: [
    "Sunday",
    "Monday",
    "Tuesday",
    "Wednesday",
    "Thursday",
    "Friday",
    "Saturday",
  ],
  es: [
    "Domingo",
    "Lunes",
    "Martes",
    "Miércoles",
    "Jueves",
    "Viernes",
    "Sábado",
  ],
} as const;

const WEEKDAYS_SHORT = {
  pt: ["Dom", "Seg", "Ter", "Qua", "Qui", "Sex", "Sáb"],
  en: ["Sun", "Mon", "Tue", "Wed", "Thu", "Fri", "Sat"],
  es: ["Dom", "Lun", "Mar", "Mié", "Jue", "Vie", "Sáb"],
} as const;

const MONTHS_FULL = {
  en: [
    "January",
    "Feburary",
    "March",
    "April",
    "May",
    "June",
    "July",
    "August",
    "September",
    "October",
    "November",
    "December",
  ],
  es: [
    "Enero",
    "Febrero",
    "Marzo",
    "Abril",
    "Mayo",
    "Junio",
    "Julio",
    "Agosto",
    "Septiembre",
    "Octubre",
    "Noviembre",
    "Diciembre",
  ],
  pt: [
    "Janeiro",
    "Fevereiro",
    "Março",
    "Abril",
    "Maio",
    "Junho",
    "Julho",
    "Agosto",
    "Setembro",
    "Outubro",
    "Novembro",
    "Dezembro",
  ],
} as const;

const MONTHS_SHORT = {
  en: [
    "Jan",
    "Feb",
    "Mar",
    "Apr",
    "May",
    "Jun",
    "Jul",
    "Aug",
    "Sep",
    "Oct",
    "Nov",
    "Dec",
  ],
  es: [
    "Ene",
    "Feb",
    "Mar",
    "Abr",
    "May",
    "Jun",
    "Jul",
    "Ago",
    "Sep",
    "Oct",
    "Nov",
    "Dic",
  ],
  pt: [
    "Jan",
    "Fev",
    "Mar",
    "Abr",
    "Mai",
    "Jun",
    "Jul",
    "Ago",
    "Set",
    "Out",
    "Nov",
    "Dez",
  ],
} as const;

const formatHourMinuteFull = (h: number, m: number, ampm: boolean) => {
  if (!ampm) return sprintf("%02d:%02d", h, m);
  const ap = h < 12 ? "AM" : "PM";
  const h2 = [0, 12].includes(h) ? 12 : h % 12;
  return sprintf("%02d:%02d %s", h2, m, ap);
};

const formatHourMinuteShort = (h: number, m: number, ampm: boolean) => {
  if (!ampm) return sprintf("%02d:%02d", h, m);
  const ap = h < 12 ? "AM" : "PM";
  const h2 = [0, 12].includes(h) ? 12 : h % 12;
  return sprintf("%d:%02d %s", h2, m, ap);
};

export const dateRecomposer = (
  timezone: Timezone,
  decomposedDateTime: DecomposedDateTime
): Date => {
  const date = new Date(decomposedDateTime.formatted);
  const secShift =
    (timezone.hr < 0 ? -1 : 1) *
    (Math.abs(timezone.hr) * 3600 + Math.abs(timezone.mn) * 60);
  date.setTime(date.getTime() - secShift * 1000);
  return date;
};

export const recomposedDateFormatter = (
  formattingStyle: FormattingStyle,
  lang: Language,
  date: Date
): string => {
  switch (formattingStyle) {
    case FormattingStyle.HoursAndMinutes:
      // "hh:mm a":en | "HH:mm" -- 08:08 AM
      return formatHourMinuteFull(
        date.getHours(),
        date.getMinutes(),
        lang == Language.en
      );
    case FormattingStyle.DayOfWeek:
      // "E" -- Mon
      return WEEKDAYS_SHORT[lang][date.getDay()];
    case FormattingStyle.DayOfMonth:
      // "d" -- 21
      return sprintf("%d", date.getDate());
    case FormattingStyle.MonthName:
      // "MMM" -- Aug
      return MONTHS_SHORT[lang][date.getMonth()];
    case FormattingStyle.LongDayOfWeekWithNumber:
      // "EEEE (d)" -- Monday (21)
      return sprintf(
        "%s (%d)",
        WEEKDAYS_FULL[lang][date.getDay()],
        date.getDate()
      );
    case FormattingStyle.DayOfWeekNumberNoParenthesis:
      // "E d" -- Mon 21
      return sprintf(
        "%s %d",
        WEEKDAYS_SHORT[lang][date.getDay()],
        date.getDate()
      );
    case FormattingStyle.TextuallyPleasantDate:
      // "MMM d (E), h:mm a":en | "d MMM (E), H:mm" -- Aug 21 (Mon), 8:10 AM
      return lang == Language.en
        ? sprintf(
            "%s %d (%s), %s",
            MONTHS_SHORT[lang][date.getMonth()],
            date.getDate(),
            WEEKDAYS_FULL[lang][date.getDay()],
            formatHourMinuteShort(date.getHours(), date.getMinutes(), true)
          )
        : sprintf(
            "%d %s (%s), %s",
            date.getDate(),
            MONTHS_SHORT[lang][date.getMonth()],
            WEEKDAYS_FULL[lang][date.getDay()],
            formatHourMinuteShort(date.getHours(), date.getMinutes(), false)
          );
    case FormattingStyle.Iso:
      // "yyyy-MM-dd'T'HH:mm:ss.SSSZ" -- 2023-08-21T20:10:21.683-0300
      return date.toISOString();
    default:
      return `<<${date.toString()}>>`;
  }
};

export const langDateFormatter = (
  formattingStyle: FormattingStyle,
  lang: Language,
  timezone: Timezone,
  decomposedDateTime: DecomposedDateTime
): string => {
  const date = dateRecomposer(timezone, decomposedDateTime);
  return recomposedDateFormatter(formattingStyle, lang, date);
};
