import Head from "next/head";
import { UseQueryResult, useQuery } from "react-query";
import axios, { AxiosResponse } from "axios";
import { OwnConfig } from "../types/OwnConfig";
import { ConventionSeries } from "../types/ConventionSeries";
import {
  AppPageState,
  AppState,
  appStateParse,
  appStateSerialize,
} from "../lib/appState";
import { Language } from "../types/Language";
import { UITranslation } from "../types/UITranslation";
import { useState } from "react";
import { Master } from "../components/Master";
import { Detail } from "../components/Detail";
import { FullScreenLoading } from "../components/FullScreenLoading";
import { ConMap } from "../components/ConMap";
import { DayNightColorPalette } from "../types/DayNightColorPalette";
import { ChangeLanguageModal } from "../components/ChangeLanguageModal";

export default function Home() {
  const [showLanguageChangeModal, setLanguageChangeModal] = useState(false);
  const [render, rerender] = useState(false);

  const conventionConfigQuery = useQuery(
    "own-config",
    async () => await axios.get<OwnConfig>("/own-config.json"),
    {}
  );
  if (conventionConfigQuery.isError) {
    window.location.reload();
  }
  const convention = conventionConfigQuery.data?.data.convention;
  const mediaPrefix = conventionConfigQuery.data?.data.media_prefix!;
  const conventionJsonUrl = conventionConfigQuery.data?.data.url;
  const UIStringQueries = {} as Record<
    Language,
    UseQueryResult<AxiosResponse<UITranslation>, unknown>
  >;
  const UIStrings = {} as Record<Language, UITranslation>;
  const languages: Language[] = Object.values(Language).sort();
  for (let language of languages) {
    // eslint-disable-next-line react-hooks/rules-of-hooks
    UIStringQueries[language] = useQuery(
      `conventionLang.${language}.${convention}`,
      async () =>
        await axios.get<UITranslation>(
          `/${convention}/strings_${language}.json`
        ),
      {}
    );
    if (UIStringQueries[language].isError) {
      window.location.reload();
    }
    UIStrings[language] = UIStringQueries[language].data?.data!;
  }
  const conventionSeriesQuery = useQuery(
    `conventionSeries.${conventionJsonUrl}`,
    async () =>
      conventionJsonUrl
        ? await axios.get<ConventionSeries>(conventionJsonUrl)
        : undefined,
    {}
  );
  const colorPaletteQuery = useQuery(
    "colorPalette",
    async () => await axios.get<DayNightColorPalette>("base-colors.json"),
    {}
  );
  if (conventionSeriesQuery.isError || colorPaletteQuery.isError) {
    window.location.reload();
  }
  const iconSizes = [57, 76, 120, 152, 167, 180, 192, 512];
  if (
    !colorPaletteQuery.isSuccess ||
    !conventionSeriesQuery.isSuccess ||
    Object.values(UIStrings).some((x) => x === undefined)
  ) {
    return (
      <>
        <Head>
          <meta charSet="UTF-8" />
          <title>Convention</title>
          <meta
            name="viewport"
            content="width=device-width, initial-scale=1.0, user-scalable=no"
          />
          <meta
            name="description"
            content={`An web app for attendees of Convention`}
          />
          <link rel="manifest" href={`/${convention}/manifest.json`} />
          {iconSizes.map((is) => (
            <link
              key={`ati.${is}`}
              rel="apple-touch-icon"
              sizes={`${is}x${is}`}
              href={`favicon_${is}.png`}
            />
          ))}
          <link rel="icon" href="/favicon.ico" />
        </Head>
        <FullScreenLoading />
      </>
    );
  }
  const changeLanguage = () => setLanguageChangeModal(!showLanguageChangeModal);
  const dayNight = window.localStorage.dayNight !== "night" ? "day" : "night";
  const flipDayNight = () => {
    window.localStorage.dayNight =
      window.localStorage.dayNight !== "night" ? "night" : "day";
    rerender(!render);
  };
  const colorPalette = colorPaletteQuery.data.data[dayNight];
  const conventionSeries = conventionSeriesQuery.data?.data!;
  const conventionEdition = conventionSeries?.editions.filter(
    (x) => x.uuid === conventionSeries.featured
  )[0]!;
  const defaultPageData: AppState = [
    conventionEdition.language,
    AppPageState.master,
    0,
    conventionSeries.featured,
  ];
  const [lang, page, tab, identifier] =
    appStateParse(window.location.hash.substring(1)) ?? defaultPageData;
  const rightHash = appStateSerialize([lang, page, tab, identifier]);
  if (window.location.hash.substring(1) !== rightHash) {
    window.location.replace(`#${rightHash}`);
  }
  const selectecEvent =
    page === AppPageState.detail
      ? conventionEdition.events.find((x) => x.uuid === identifier)
      : undefined;
  if (
    !Object.values(AppPageState).includes(page) ||
    (page === AppPageState.detail && selectecEvent === undefined)
  ) {
    window.location.replace(`#${appStateSerialize(defaultPageData)}`);
    rerender(!render);
  }
  /*
  conventionSeries.urgent_message = {
    uuid: "asd",
    card: {
      name: { alias: "x", uuid: "x", en: "hfgh", es: "hfgh", pt: "hfgh" },
      description: { alias: "x", uuid: "x", en: "fbg", es: "fbg", pt: "fbg" },
      icon: "ic_warning",
    },
    tab: {
      name: { alias: "x", uuid: "x", en: "asd", es: "asd", pt: "asd" },
      description: { alias: "x", uuid: "x", en: "dsf", es: "dsf", pt: "dsf" },
      text: { alias: "x", uuid: "x", en: "zxc", es: "zxc", pt: "zxc" },
    },
  };
  //*/
  const Child =
    page === AppPageState.master
      ? () => (
          <Master
            UIStrings={UIStrings}
            conventionSeries={conventionSeries}
            conventionEdition={conventionEdition}
            mediaPrefix={mediaPrefix}
            flipDayNight={flipDayNight}
            dayNight={dayNight}
            rerender={() => rerender(!render)}
            changeLanguage={changeLanguage}
          />
        )
      : page === AppPageState.conMap
      ? () => (
          <ConMap
            UIStrings={UIStrings}
            conventionEdition={conventionEdition}
            mediaPrefix={mediaPrefix}
          />
        )
      : () => (
          <Detail
            UIStrings={UIStrings}
            lang={lang}
            conventionEdition={conventionEdition}
            mediaPrefix={mediaPrefix}
            selectedEvent={selectecEvent!}
            lightDarkColors={[]}
          />
        );
  return (
    <>
      <Head>
        <meta charSet="UTF-8" />
        <title>{conventionEdition.name[lang]}</title>
        <meta
          name="viewport"
          content="width=device-width, initial-scale=1.0, user-scalable=no"
        />
        <meta
          name="description"
          content={`An web app for attendees of ${conventionEdition.name[lang]}`}
        />
        <link rel="manifest" href={`/${convention}/manifest.json`} />
        {iconSizes.map((is) => (
          <link
            key={`ati.${is}`}
            rel="apple-touch-icon"
            sizes={`${is}x${is}`}
            href={`favicon_${is}.png`}
          />
        ))}
        <link rel="icon" href="/favicon.ico" />
        <style>
          :root&#123;--primary_darker:
          {conventionEdition.color_scheme.primary_darker};--primary_regular:
          {conventionEdition.color_scheme.primary_regular};--primary_lighter:
          {conventionEdition.color_scheme.primary_lighter};--accent_color:
          {conventionEdition.color_scheme.accent_color};&#125;
        </style>
        <style>
          :root&#123;
          {Object.entries(colorPalette)
            .map(([key, value]) => `--${key}:${value};`)
            .join("")}
          &#125;
        </style>
      </Head>
      <Child />
      <ChangeLanguageModal
        show={showLanguageChangeModal}
        close={() => setLanguageChangeModal(false)}
        UIStrings={UIStrings}
        lang={lang}
        conventionSeries={conventionSeries}
        rerender={() => rerender(!render)}
      />
    </>
  );
}
